@@ -1,8 +1,7 @@
 # coding=utf-8
-from __future__ import print_function
+import functools
 import logging
 from uuid import uuid4
-from functools import wraps
 
 from ._compat import to_unicode
 
@@ -18,6 +17,8 @@ class AuthorizationMixin(object):
             logger.debug(u'New CSFR token')
             csrf_token = self.make_csrf_token()
             session[self.csrf_key] = csrf_token
+            if callable(getattr(session, 'save', None)):
+                session.save()
         return csrf_token
 
     def make_csrf_token(self):
@@ -58,11 +59,10 @@ class AuthorizationMixin(object):
         roles = [to_unicode(r) for r in roles]
 
         def decorator(f):
-            @wraps(f)
+            @functools.wraps(f)
             def wrapper(*args, **kwargs):
                 logger = logging.getLogger(__name__)
-                request = options.get('request') or self.request or \
-                    args and args[0]
+                request = options.get('request') or self.request or args and args[0]
                 url_sign_in = self._get_url_sign_in(request, options)
 
                 user = self.get_user()
@@ -82,7 +82,7 @@ class AuthorizationMixin(object):
                         return self.wsgi.raise_forbidden()
 
                 disable_csrf = csrf == False  # noqa
-                if (self.wsgi.not_safe_method(request) and not disable_csrf) or csrf:
+                if (not self.wsgi.is_idempotent(request) and not disable_csrf) or csrf:
                     if not self.csrf_token_is_valid(request):
                         logger.debug(u'User `{0}`: invalid CSFR token'.format(user.login))
                         return self.wsgi.raise_forbidden("CSFR token isn't valid")
@@ -101,6 +101,8 @@ class AuthorizationMixin(object):
 
     def _login_required(self, request, url_sign_in):
         self.session[self.redirect_key] = self.wsgi.get_full_path(request)
+        if callable(getattr(self.session, 'save', None)):
+            self.session.save()
         return self.wsgi.redirect(url_sign_in)
 
     def _get_url_sign_in(self, request, options):
